
%%        
% constructor, connects to the board and creates an arduino object
function a = arduino(comPort)

comPort = 'COM3';    

% check whether serial port is currently used by MATLAB
if ~isempty(instrfind({'Port'}, {comPort}))
    
    disp(['The port ' comPort ' is already used by MATLAB']);
    disp(['If you are sure that the board is connected to ' comPort]);
    disp('then delete the object, execute:');
    disp(['  delete(instrfind({''Port''},{''' comPort '''}))']);
    disp('to delete the port, disconnect the cable, reconnect it,');
    disp('and then create a new arduino object');
    error(['Port ' comPort ' already used by MATLAB']);
    
else
    
    % define serial object
    a = serial(comPort, 'BaudRate', 115200);

    % open port
    try
        fopen(a);

        % it takes several seconds before any operation could be attempted
        fprintf(1,'Attempting connection .');
        for i = 1:3
            pause(1);
            fprintf(1,'.');
        end
        fprintf(1,'\n');

        % flush serial buffer before sending anything
        flush(a);

        % query sketch type
        fwrite(a, [57 57], 'uchar');
        sketch = fscanf(a,'%d');

        % exit if there was no answer
        if isempty(sketch)
            delete(a);
            error('Connection unsuccessful, please make sure that the board is powered on, running a sketch provided with the package, and connected to the indicated serial port. You might also try to unplug and re-plug the USB cable before attempting a reconnection.');
        end

        % check returned value
        if sketch == 0
            disp('Basic Analog and Digital I/O (adio.ino) sketch detected !');
        elseif sketch == 1
            disp('Analog & Digital I/O + Encoders (adioe.ino) sketch detected !');
        elseif sketch == 2
            disp('Analog & Digital I/O + Encoders + Servos (adioes.ino) sketch detected !');
        elseif sketch == 3
            disp('Motor Shield V1 (plus adioes.ino functions) sketch detected !');
        elseif sketch == 4
            disp('Motor Shield V2 (plus adioes.ino functions) sketch detected !');
        elseif sketch == 5
            disp('Basic Analog and Digital I/O (adio.ino functions) + audio sketch detected!');
        else
            delete(a);
            error('Unknown sketch. Please make sure that a sketch provided with the package is running on the board');
        end

        % notify successful installation
        disp('Arduino successfully connected !');

        setappdata(gcf, 'SerialObject', a);
        
    catch 
        delete(a);
        error(['Could not open port: ' comPort]);
    end
    
end
%%        
% distructor, deletes the object
function delete(a)

% Use delete(a) or a.delete to delete the arduino object

% if it is a serial, valid and open then close it
if isa(a, 'serial') && isvalid(a) && strcmpi(get(a, 'Status'), 'open'),
        try
            % trying to leave it in a known unharmful state
            for i = 2:69
                a = pinMode(a, i, 'output');
                digitalWrite(a, i, 0);
                a = pinMode(a, i,'input');
            end
        catch
            % disp but proceed anyway
            disp('Could not sucesfully turn everything off.');
            disp('Proceeding to deletion anyway');
        end
    fclose(a);
end

% if it's an object delete it
if isobject(a),
    delete(a);
end

clc;
disp('Arduino successfully disconnected!');


%%        
% disp, displays the object
function disp(a)

% disp(a) or a.disp, displays the arduino object properties
% The first and only argument is the arduino object, there is no
% output, but the basic information and properties of the arduino
% object are displayed on the screen.
% This function is called when just the name of the arduino object
% is typed on the command line, followed by enter. The command
% str=evalc('a.disp'), (or str=evalc('a')), can be used to capture
% the output in the string 'str'.

if isvalid(a),
    if isa(a.aser,'serial') && isvalid(a.aser),
        disp(['<a href="matlab:help arduino">arduino</a> object connected to ' a.aser.port ' port']);
        if a.sktc==4,
            disp('Motor Shield sketch V2 (plus adioes.pde functions) running on the board');
            disp(' ');
            a.servoStatus
            disp(' ');
            disp('Servo Methods: <a href="matlab:help servoStatus">servoStatus</a> <a href="matlab:help servoAttach">servoAttach</a> <a href="matlab:help servoDetach">servoDetach</a> <a href="matlab:help servoRead">servoRead</a> <a href="matlab:help servoWrite">servoWrite</a>');
            disp(' ');
            a.encoderStatus
            disp(' ');
            disp('Encoder Methods: <a href="matlab:help encoderStatus">encoderStatus</a> <a href="matlab:help encoderAttach">encoderAttach</a> <a href="matlab:help encoderDetach">encoderDetach</a> <a href="matlab:help encoderRead">encoderRead</a> <a href="matlab:help encoderReset">encoderReset</a>');
            disp(' ');
            a.motorSpeed
            a.stepperSpeed
            disp(' ');
            disp('DC Motor and Steppers Methods: <a href="matlab:help motorSpeed">motorSpeed</a> <a href="matlab:help motorRun">motorRun</a> <a href="matlab:help stepperSpeed">stepperSpeed</a> <a href="matlab:help stepperStep">stepperStep</a>');
            disp(' ');
            disp('Serial port and other Methods: <a href="matlab:help serial">serial</a> <a href="matlab:help flush">flush</a> <a href="matlab:help roundTrip">roundTrip</a>');
        elseif a.sktc==3,
            disp('Motor Shield sketch V1 (plus adioes.pde functions) running on the board');
            disp(' ');
            a.servoStatus
            disp(' ');
            disp('Servo Methods: <a href="matlab:help servoStatus">servoStatus</a> <a href="matlab:help servoAttach">servoAttach</a> <a href="matlab:help servoDetach">servoDetach</a> <a href="matlab:help servoRead">servoRead</a> <a href="matlab:help servoWrite">servoWrite</a>');
            disp(' ');
            a.encoderStatus
            disp(' ');
            disp('Encoder Methods: <a href="matlab:help encoderStatus">encoderStatus</a> <a href="matlab:help encoderAttach">encoderAttach</a> <a href="matlab:help encoderDetach">encoderDetach</a> <a href="matlab:help encoderRead">encoderRead</a> <a href="matlab:help encoderReset">encoderReset</a>');
            disp(' ');
            a.motorSpeed
            a.stepperSpeed
            disp(' ');
            disp('DC Motor and Steppers Methods: <a href="matlab:help motorSpeed">motorSpeed</a> <a href="matlab:help motorRun">motorRun</a> <a href="matlab:help stepperSpeed">stepperSpeed</a> <a href="matlab:help stepperStep">stepperStep</a>');
            disp(' ');
            disp('Serial port and other Methods: <a href="matlab:help serial">serial</a> <a href="matlab:help flush">flush</a> <a href="matlab:help roundTrip">roundTrip</a>');
        elseif a.sktc==2,
            disp('Analog & Digital I/O + Encoders + Servos (adioes.pde) sketch running on the board');
            disp(' ');
            a.pinMode
            disp(' ');
            disp('Pin IO Methods: <a href="matlab:help pinMode">pinMode</a> <a href="matlab:help digitalRead">digitalRead</a> <a href="matlab:help digitalWrite">digitalWrite</a> <a href="matlab:help analogRead">analogRead</a> <a href="matlab:help analogWrite">analogWrite</a> <a href="matlab:help analogReference">analogReference</a>');
            disp(' ');
            a.servoStatus
            disp(' ');
            disp('Servo Methods: <a href="matlab:help servoStatus">servoStatus</a> <a href="matlab:help servoAttach">servoAttach</a> <a href="matlab:help servoDetach">servoDetach</a> <a href="matlab:help servoRead">servoRead</a> <a href="matlab:help servoWrite">servoWrite</a>');
            disp(' ');
            a.encoderStatus
            disp(' ');
            disp('Encoder Methods: <a href="matlab:help encoderStatus">encoderStatus</a> <a href="matlab:help encoderAttach">encoderAttach</a> <a href="matlab:help encoderDetach">encoderDetach</a> <a href="matlab:help encoderRead">encoderRead</a> <a href="matlab:help encoderReset">encoderReset</a>');
            disp(' ');
            disp('Serial port and other Methods: <a href="matlab:help serial">serial</a> <a href="matlab:help flush">flush</a> <a href="matlab:help roundTrip">roundTrip</a>');
        elseif a.sktc==1,
            disp('Analog & Digital I/O + Encoders (adioe.pde) sketch running on the board');
            disp(' ');
            a.pinMode
            disp(' ');
            disp('Pin IO Methods: <a href="matlab:help pinMode">pinMode</a> <a href="matlab:help digitalRead">digitalRead</a> <a href="matlab:help digitalWrite">digitalWrite</a> <a href="matlab:help analogRead">analogRead</a> <a href="matlab:help analogWrite">analogWrite</a> <a href="matlab:help analogReference">analogReference</a>');
            disp(' ');
            a.encoderStatus
            disp(' ');
            disp('Encoder Methods: <a href="matlab:help encoderStatus">encoderStatus</a> <a href="matlab:help encoderAttach">encoderAttach</a> <a href="matlab:help encoderDetach">encoderDetach</a> <a href="matlab:help encoderRead">encoderRead</a> <a href="matlab:help encoderReset">encoderReset</a>');
            disp(' ');
            disp('Serial port and other Methods: <a href="matlab:help serial">serial</a> <a href="matlab:help flush">flush</a> <a href="matlab:help roundTrip">roundTrip</a>');
        else
            disp('Basic Analog & Digital I/O sketch (adio.pde) running on the board');
            disp(' ');
            a.pinMode
            disp(' ');
            disp('Pin IO Methods: <a href="matlab:help pinMode">pinMode</a> <a href="matlab:help digitalRead">digitalRead</a> <a href="matlab:help digitalWrite">digitalWrite</a> <a href="matlab:help analogRead">analogRead</a> <a href="matlab:help analogWrite">analogWrite</a> <a href="matlab:help analogReference">analogReference</a>');
            disp(' ');
            disp('Serial port and other Methods: <a href="matlab:help serial">serial</a> <a href="matlab:help flush">flush</a> <a href="matlab:help roundTrip">roundTrip</a>');
        end
        disp(' ');
    else
        disp('<a href="matlab:help arduino">arduino</a> object connected to an invalid serial port');
        disp('Please delete the arduino object');
        disp(' ');
    end
else
    disp('Invalid <a href="matlab:help arduino">arduino</a> object');
    disp('Please clear the object and instantiate another one');
    disp(' ');
end

%%        
% serial, returns the serial port
function str = serial(a)

% serial(a) (or a.serial), returns the name of the serial port
% The first and only argument is the arduino object, the output
% is a string containing the name of the serial port to which
% the arduino board is connected (e.g. 'COM9', 'DEMO', or
% '/dev/ttyS101'). The string 'Invalid' is returned if
% the serial port is invalid

if isvalid(a.aser),
    str = a.aser.port;
else
    str = 'Invalid';
end

%%
% flush, clears the pc's serial port buffer
function val = flush(a)

% val=flush(a) (or val=a.flush) reads all the bytes available 
% (if any) in the computer's serial port buffer, therefore 
% clearing said buffer.
% The first and only argument is the arduino object, the 
% output is a vector of bytes that were still in the buffer.
% The value '-1' is returned if the buffer was already empty.

val = -1;
if a.BytesAvailable > 0
    val = fread(a, a.BytesAvailable);
end

%%        
% pin mode, changes pin mode
function pinMode(a, pin, str)

    % pinMode(a,pin,str); reads or sets the I/O mode of a digital pin.
    % The first argument, a, is the arduino object.
    % The second argument, pin, is the number of the digital pin (2 to 69).
    % The third argument, str, is a string that can be 'input' or 'output',
    % Called as pinMode(a,pin) it returns the mode of the digital pin, 
    % called as pinMode(a), it prints the mode of all the digital pins. 
    % Note that in the Arduino Uno board the digital pins from 0 to 13 
    % are located on the upper right part of the board,
    % while the digital pins from 14 to 19 are better known as
    % "analog input" pins (in fact are often referred to with an 
    % analog pin number from 0 to 5) and are located in the lower 
    % right corner of the board.
    %
    % Examples:
    % pinMode(a,11,'output') % sets digital pin #11 as output
    % pinMode(a,10,'input')  % sets digital pin #10 as input
    % a.pinMode(10,'input')  % same as pinMode(a,10,'input')
    % val=pinMode(a,10);     % returns the status of digital pin #10
    % pinMode(a,5);          % prints the status of digital pin #5
    % pinMode(a);            % prints the status of all pins
    %

% assign value
if lower(str(1)) == 'o'
    val = 1; 
else
    val = 0; 
end

% send mode, pin and value
fwrite(a, [48 97+pin 48+val], 'uchar');

% silently detach servos on the correspinding pin
servoDetach(a, pin);
        
%%        
% digital read
function val = digitalRead(a, pin)

% val=digitalRead(a,pin); performs digital input on a given arduino pin.
% The first argument, a, is the arduino object.
% The second argument, pin, is the number of the digital pin (2 to 69)
% where the digital input needs to be performed. On the Arduino Uno
% the digital pins from 0 to 13 are located on the upper right part
% while the digital pins from 14 to 19 are better known as "analog input"
% pins and are located in the lower right corner of the board
% (in fact are often referred to as "analog pins from 0 to 5").
%
% Examples:
% val=digitalRead(a,4); % reads pin #4
% val=a.digitalRead(4); % just as above (reads pin #4)
%

% send mode and pin
fwrite(a, [49 97+pin], 'uchar');

% get value
val = fscanf(a, '%d');
                
%%        
% digital write
function digitalWrite(a, pin, val)

% digitalWrite(a,pin,val); performs digital output on a given pin.
% The first argument, a, is the arduino object.
% The second argument, pin, is the number of the digital pin 
% (2 to 69) where the digital output value needs to be written.
% The third argument, val, is the output value (either 0 or 1).
% On the Arduino Uno  the digital pins from 0 to 13 are located
% on the upper right part of the board, while the digital pins
% from 14 to 19 are better known as "analog input" pins and are
% located in the lower right corner of the board  (in fact are 
% often referred to as "analog pins from 0 to 5").
%
% Examples:
% digitalWrite(a,13,1); % sets pin #13 high
% digitalWrite(a,13,0); % sets pin #13 low
% a.digitalWrite(13,0); % just as above (sets pin #13 to low)
%

% send mode, pin and value
fwrite(a, [50 97+pin 48+val], 'uchar');

        
%%        
% analog read
function val = analogRead(a, pin)

% val=analogRead(a,pin); Performs analog input on a given arduino pin.
% The first argument, a, is the arduino object. The second argument, 
% pin, is the number of the analog input pin (0 to 15) from which the 
% analog value needs to be read. The returned value, val, ranges from 
% 0 to 1023, with 0 corresponding to an input voltage of 0 volts,
% and 1023 to a reference value that is typically 5 volts (this voltage can
% be set up by the analogReference function). Therefore, assuming a range
% from 0 to 5 V the resolution is .0049 volts (4.9 mV) per unit.
% Note that in the Arduino Uno board the analog input pins 0 to 5 are also
% the digital pins from 14 to 19, and are located on the lower right corner.
% Specifically, analog input pin 0 corresponds to digital pin 14, and analog
% input pin 5 corresponds to digital pin 19. Performing analog input does
% not affect the digital state (high, low, digital input) of the pin.
%
% Examples:
% val=analogRead(a,0); % reads analog input pin # 0
% val=a.analogRead(0); % just as above, reads analog input pin # 0
%

% send mode and pin
fwrite(a, [48+3 97+pin], 'uchar');

% get value
val = fscanf(a, '%d');


%%        
% function analog write
function analogWrite(a, pin, val)

% analogWrite(a,pin,val); Performs analog output on a given arduino pin.
% The first argument, a, is the arduino object. The second argument, 
% pin, is the number of the DIGITAL pin where the analog (PWM) output 
% needs to be performed. Allowed pins for AO on the Mega board
% are 2 to 13 and 44 to 46, (3,5,6,9,10,11 on the Uno board).
% The second argument, val, is the value from 0 to 255 for the level of
% analog output. Note that the digital pins from 0 to 13 are located on the
% upper right part of the board.
%
% Examples:
% analogWrite(a,11,90); % sets pin #11 to 90/255
% a.analogWrite(3,10); % sets pin #3 to 10/255
%

% send mode, pin and value
fwrite(a,[52 97+pin val],'uchar');

%%        
% function analog reference
function analogReference(a, str)

% analogReference(a,str); Changes voltage reference on analog input pins
% The first argument, a, is the arduino object. The second argument, 
% str, is one of these strings: 'default', 'internal' or 'external'. 
% This sets the reference voltage used at the top of the input ranges.
%
% Examples:
% analogReference(a,'default'); % sets default reference
% analogReference(a,'internal'); % sets internal reference
% analogReference(a,'external'); % sets external reference
% a.analogReference('external'); % just as above (sets external reference)
%

if lower(str(1)) == 'e'
    num = 2;
elseif lower(str(1)) == 'i'
    num = 1;
else
    num = 0;
end

% send mode, pin and value
fwrite(a, [82 48+num], 'uchar');


%%        
% servo attach
function servoAttach(a, pin)

% servoAttach(a,pin); or a.servoAttach(pin); attaches a servo to a pin.
% The first argument, a, is the arduino object. The second argument, 
% pin, is the number of the pwm pin where the servo must be attached 
% (a servo has three pins, a central red one which goes to +5V,
% a black or brown one which goes to ground, and a white or orange one 
% which must be connected to an analog output (PWM) pin on the Arduino).
%
% Earlier versions of both Arduino boards and Arduino IO package supported
% only pwm pins 9 and 10, now up to 12 different pins are supported on most
% Arduino boards, and 48 on the Mega. Note that on boards other than the Mega,
% use of the servos disables analogWrite() functionality on pins 9 and 10,
% whether or not there is a Servo on those pins. On the Mega, up to 12 servos
% can be used without interfering with PWM; use of 12 to 23 servos will
% disable PWM on pins 11 and 12.
% 
% Note that motor shields typically have their own servo connectors,
% For example the Adafruit Motor Shield V1 has 2 servo connector on
% its top left corner, the one labeled "SER1" uses pin #10, 
% while the one labeled "SERVO_2" uses the pin #9.
% 
% Examples:
% servoAttach(a,10); % attaches servo on pin #10
% a.servoAttach(10); % same as above (attaches servo on pin #10)
%

% send mode, pin and value (1 for attach)
fwrite(a, [54 97+pin 48+1], 'uchar');


%%        
% servo detach
function servoDetach(a ,pin)

% servoDetach(a,pin); detaches a servo from its corresponding pwm pin.
% The first argument, a, is the arduino object. The second argument, 
% pin, is the number of the pin where the servo is attached
% Earlier versions supported only pins 9 and 10, now up to 12 different 
% pins are supported on most Arduino boards, and 48 on the Mega. 
% Note that on boards other than the Mega, using the servos disables 
% analogWrite() functionality on pins 9 and 10, whether or not there 
% is a Servo on those pins. On the Mega, up to 12 servos can be used 
% without interfering with PWM; use of 12 to 23 servos will 
% disable PWM on pins 11 and 12.            
%
% Examples:
% servoDetach(a,10); % detaches a servo attached on pin 10
% a.servoDetach(10); % detaches a servo attached on pin 10
%

% send mode, pin and value (0 for detach)
fwrite(a, [54 97+pin 48+0], 'uchar');


%%        
% servo status
function val = servoStatus(a, pin)

% servoStatus(a,pin); Reads the status of a servo (attached/detached).
% The first argument, a, is the arduino object. The second argument, 
% pin, is the number of the pin where the servo is attached
% Earlier versions supported only pins 9 and 10, now up to 12 different 
% pins are supported on most Arduino boards, and 48 on the Mega.
% The returned value is either 1 (servo attached) or 0 (servo detached).
% Called without output arguments, the function prints a string specifying
% the status of the servo. Called without input arguments, the function
% either returns the status vector or prints the status of each servo.
%
% Examples:
% val=servoStatus(a,10); % return the status of servo on pin #10
% servoStatus(a,10); % prints the status of servo on pin #10
% servoStatus(a); % prints the status of all servos
% a.servoStatus; % prints the status of all servos
%

% with no arguments calls itself recursively for all servos
if nargin == 1
    if nargout > 0
        val = zeros(69,1);
        for i = 2:69
            val(i) = servoStatus(a, i);
        end
        return
    else
        for i = 2:69
            servoStatus(a, i);
        end
        return
    end
end

% send mode and pin
fwrite(a, [53 97+pin], 'uchar');

% get value
val = fscanf(a, '%d');

%%        
% servo read
function val = servoRead(a, pin)

% val=servoRead(a,pin); reads the angle of a given servo.
% The first argument, a, is the arduino object.
% The second argument, pin, is the number of the pin where the servo is attached
% Earlier versions supported only pins 9 and 10, now up to 12 different pins are
% supported on most Arduino boards, and 48 on the Mega.
% The returned value is the angle in degrees, typically from 0 to 180.
% 
% Note that this function returns random values in DEMO mode, 
% or when a sketch that does not support servos is running on 
% the Arduino board.
%
% Examples:
% val=servoRead(a,10); % reads angle from servo on pin #10
% val=a.servoRead(9); % reads angle from servo on pin #9
%

% send mode and pin
fwrite(a, [55 97+pin], 'uchar');

%%        
% servo write
function servoWrite(a, pin, val)

% servoWrite(a,pin,val); writes an angle on a given servo.
% The first argument, a, is the arduino object.
% The second argument, pin, is the number of the pin where the servo is attached
% Earlier versions supported only pins 9 and 10, now up to 12 different pins are
% supported on most Arduino boards, and 48 on the Mega.
% The third argument is the angle in degrees, typically from 0 to 180.
% 
% Examples:
% servoWrite(a,10,45); % rotates servo on pin #10 to 45 degrees
% a.servoWrite(10,70); % rotates servo on pin #9 to 70 degrees
%

% send mode, pin and value
fwrite(a, [56 97+pin val], 'uchar');

%%        
% encoder attach
function encoderAttach(a, enc, pinA, pinB)

% encoderAttach(a,enc,pinA,pinB); attaches an encoder to 2 pins.
% The first argument, a, is the arduino object.
% The second argument is the encoder's number, either 0,1 or 2.
% The third and fourth arguments, pinA and pinB, are the number of 
% the pins where the encoder's pinA and pinB are attached (the
% "common" pin a.k.a. pinC should be attached to the ground).
% 
% Note that, since this methods relies on attaching interrupt
% service routines to the specified pins, the allowed pins for
% encoder attachment are only 2,3,18,19,20,21 (see also
% http://arduino.cc/it/Reference/AttachInterrupt for details).
% Also note that, to avoid interferences, this function disables
% any output and servo functionality for the attached pins.
%
% Examples:
% encoderAttach(a,0,2,3); % attaches encoder #0 on pins 2 and 3
% a.encoderAttach(1,18,19); % attaches encoder #1 on pins 18 and 19
%

% silently detach servos on the correspinding pins
servoDetach(a, pinA);
servoDetach(a, pinB);

% send mode, encoder and pins
fwrite(a, [69 48+enc 97+pinA 97+pinB], 'uchar');

%%        
% encoder detach
function encoderDetach(a, enc)

% encoderDetach(a,enc); detaches an encoder from its pins.
% The first argument before the function name, a, is the arduino object.
% The second argument, enc, is the number of the encoder to
% be detached (0, 1, or 2). This also detaches any interrupt
% routine previously attached to the specified pins.
%
% Examples:
% encoderDetach(a,0); % detach encoder #0
% a.encoderDetach(1); % detach encoder #1
%

% send mode, encoder and value
fwrite(a, [70 48+enc], 'uchar');

%%        
% encoder status
function val = encoderStatus(a, enc)

% encoderStatus(a,enc); Reads the status of an encoder (attached/detached).
% The first argument, a, is the arduino object. The second argument, enc, 
% is the number of the encoder (0 to 2) which status needs to be read.
% The returned value is either 1 (encoder attached) or 0 (encoder detached).
% Called without output arguments, the function prints a string specifying
% the status of the encoder. Called without input arguments, the function
% either returns the status vector or prints the status of each encoder.
%
% Examples:
% val=encoderStatus(a,0); % return the status of encoder #0
% encoderStatus(a,2); % prints the status of encoder #2
% encoderStatus(a); % prints the status of all encoders
% a.encoderStatus; % prints the status of all encoders
%

% with no arguments calls itself recursively for all encoders
if nargin == 1
    if nargout > 0
        val = zeros(3,1);
        for i = 0:2
            val(i+1) = encoderStatus(a, i);
        end
        return
    else
        for i = 0:2
            encoderStatus(a, i);
        end
        return
    end
end

% gets value from the encoder state vector
val = a.encs(1+enc);

%%        
% encoder read
function val = encoderRead(a, enc)

% val=encoderRead(a,enc); reads the position of a given encoder.
% The first argument, a, is the arduino object.
% The second argument, enc, is the encoder number (0 to 2).
% The returned value is the position in steps, where clockwise
% rotation are assumed positive by convention.
% 
% Note that this function returns random values in DEMO mode, 
% or when a sketch that does not support encoders is running on 
% the Arduino board.
%
% Examples:
% val=encoderRead(a,2); % reads angle from encoder #2
% val=a.encoderRead(0); % reads angle from encoder #0
%

% send mode and enc
fwrite(a, [71 48+enc], 'uchar');

%%        
% encoder reset
function encoderReset(a, enc)

% encoderReset(a,enc); resets the position of a given encoder.
% The first argument, a, is the arduino object.
% The second argument, enc, is the number of the encoder (0 to 2).
%
% Examples:
% encoderReset(a,1); % resets encoder #1
% a.encoderReset(2); % resets encoder #2
%

% send mode and encoder number
fwrite(a, [72 48+enc], 'uchar');

%%        
% encoder attach
function encoderDebounce(a, enc, del)

% encoderDebounce(a,enc,del); sets debounce delay for an encoder.
% The first argument, a, is the arduino object.
% The second argument is the encoder's number, either 0,1 or 2.
% The third argument, del, is the debounce delay (in units of
% approximately 0.1 ms each) between the instant in which an 
% interrupt is triggered on either pinA or pinB and the instant
% in which the status (HIGH or LOW) of the corresponding pin is
% actually read to understand the rotation direction.
% 
% Note that this delay will limit the maximum rotation rate that
% can be detected, so use it only if you know what you are doing.
% Possible values go from 0 (no delay) to 69 (6.9 ms). In general, 
% it is suggested to keep this value no higher than 20 (2 ms). 
%
% Examples:
% encoderDebounce(a,0,20); % sets a delay of 20 for encoder #0
% a.encoderDebounce(1,17); % sets a delay of 17 for encoder #1
%

% send mode, encoder and pins
fwrite(a, [73 48+enc 97+del], 'uchar');

%%
% round trip
function val = roundTrip(a, byte)

% roundTrip(a,byte); sends something to the arduino and back
% The first argument, a, is the arduino object.
% The second argument, byte, is any integer from 0 to 255.
% The output is the same byte, which was received from the
% arduino and sent back along the serial connection unchanged.
%
% This is provided as an example for people that want to add
% their own code to this arduino class (the section handling
% this dummy function in the pde file is handled as "case 400:",
% one might take the parameter, perform some potentially useful
% operation, and then send any result back via serial connection).
%
% Examples:
% roundTrip(a,48); % sends '48' to the arduino and back.
% a.roundTrip(53); % sends '53' to the arduino and back.
%

% send mode and byte
fwrite(a, [88 byte], 'uchar');

% get value back
val = fscanf(a, '%d');

%%        
% motor speed
function val = motorSpeed(a, num, val)

% val=motorSpeed(a,num,val); sets the speed of a DC motor.
% The first argument, a, is the arduino object.
% The second argument, num, is the number of the motor, which can go
% from 1 to 4 (the motor ports are numbered on the motor shield).
% The third argument is the speed from 0 (stopped) to 255 (maximum), note
% that depending on the motor speeds of at least 60 might be necessary
% to actually run it. Called as motorSpeed(a,num), it returns 
% the speed at which the given motor is set to run. If there
% is no output argument it prints the speed of the motor.
% Called as motorSpeed(a), it prints the speed of each motor.
% Note that you must use the motorRun function to actually run
% the motor at the given speed, either forward or backwards.
% Note that this function returns random values in DEMO mode, 
% or when a sketch that does not support the Adafruit motor
% shield is running on the Arduino board.
%
% Examples:
% motorSpeed(a,4,200)      % sets speed of motor 4 as 200/255
% val=motorSpeed(a,1);     % returns the speed of motor 1
% motorSpeed(a,3);         % prints the speed of motor 3
% motorSpeed(a);           % prints the speed of all motors
% a.motorSpeed;            % prints the speed of all motors
%

% send mode, num and value
fwrite(a, [65 48+num val], 'uchar');

%%        
% motor run
function motorRun(a, num, dir)

% motorRun(a,num,dir); runs a given DC motor.
% The first argument, a, is the arduino object.
% The second argument, num, is the number of the motor, which can go
% from 1 to 4 (the motor ports are numbered on the motor shield).
% The third argument, dir, should be a string that can be 'forward'
% (runs the motor forward) 'backward' (runs the motor backward)
% or 'release', (stops the motor). Note that since version 3.0,
% a +1 is interpreted as 'forward', a 0 is interpreted
% as 'release', and a -1 is interpreted as 'backward'.
%
% Examples:
% motorRun(a,1,'forward');      % runs motor 1 forward
% motorRun(a,3,'backward');     % runs motor 3 backward
% motorRun(a,2,-1);             % runs motor 2 backward
% motorRun(a,1,'release');      % releases motor 1
% a.motorRun(2,'release');      % releases motor 2
%

% allows for direction to be set by 1, 0, -1
if isnumeric(dir) && isscalar(dir),
    switch dir
        case 1
            dir = 'forward';
        case 0
            dir = 'release';
        case -1
            dir = 'backward';
    end
end

% send mode, num and value
fwrite(a, [66 48+num abs(dir(1))], 'uchar');


%%        
% stepper speed
function val = stepperSpeed(a, num, val)

% val=stepperSpeed(a,num,val); sets the speed of a given stepper motor
% The first argument, a, is the arduino object.
% The second argument, num, is the number of the stepper motor,
% which can go from 1 to 4 (the motor ports are numbered on the motor shield).
% The third argument is the RPM speed from 1 (minimum) to 255 (maximum).
% Called as stepperSpeed(a,num), it returns the speed at which 
% the given motor is set to run. If there is no output
% argument it prints the speed of the stepper motor.
% Called as stepperSpeed(a), it prints the speed of each stepper motor.
% Note that you must use the stepperStep function to actually run
% the motor at the given speed, either forward or backwards (or release
% it). Note that this function returns random values in DEMO mode, 
% or when a sketch that does not support the Adafruit motor
% shield is running on the Arduino board.
%
% Examples:
% stepperSpeed(a,2,50)      % sets speed of stepper 2 as 50 rpm
% val=stepperSpeed(a,1);    % returns the speed of stepper 1
% stepperSpeed(a,2);        % prints the speed of stepper 2
% stepperSpeed(a);          % prints the speed of both steppers
% a.stepperSpeed;           % prints the speed of both steppers
%

% send mode, num and value
fwrite(a, [48+19 48+num val], 'uchar');

%%        
% stepper step
function stepperStep(a, num, dir, sty, steps)

% stepperStep(a,num,dir,sty,steps); rotates a given stepper motor
% The first argument, a, is the arduino object. The second argument, 
% num, is the number of the stepper motor, which is either 1 or 2. 
% The third argument, the direction, is a string that can be 'forward'
% (runs the motor forward), 'backward' (runs the motor backward),
% or 'release', (stops and releases the motor). Note that since version 3.0,
% a +1 is interpreted as 'forward', a 0 is interpreted as 'release',
% and a -1 is interpreted as 'backward'. Unless the direction is 'release',
% then two more argument are needed: the fourth one is the style,
% which is a string specifying the style of the motion, and can be 'single'
% (only one coil activated at a time), 'double' (2 coils activated, gives
% an higher torque and power consumption) 'interleave', (alternates between
% single and double to get twice the resolution and half the speed), and
% 'microstep' (the coils are driven in PWM for a smoother motion).
% The final argument is the number of steps that the motor has
% to complete.
%
% Examples:
% % rotates stepper 1 forward of 100 steps in interleave mode
% stepperStep(a,1,'forward','double',100);
% % rotates stepper 2 forward of 50 steps in double mode
% stepperStep(a,1,'forward','double',50);
% % rotates stepper 2 backward of 50 steps in single mode
% stepperStep(a,2,'backward','single',50);
% % rotates stepper 2 backward of 50 steps in interleave mode
% a.stepperStep(2,'backward','interleave',50);
%

% allows for direction to be set by 1,0,-1
if isnumeric(dir) && isscalar(dir)
    switch dir
        case 1
            dir = 'forward';
        case 0
            dir = 'release';
        case -1
            dir = 'backward';
    end
end

% send mode, num and value
fwrite(a, [68 48+num abs(dir(1)) abs(sty(1)) steps], 'uchar');

